/*
+---------------------------------------------------------------------------------------+
|																						|
|	Author: Jonas Eberhard | www.joseng.de												|
|   Originalfunktion: Ulrich Radig | www.ulrichradig.de									|
|	Datei:	dmx.c																		|
|	Beschreibung: DMX Empfangsroutine  													|
|																						|
|																						|
|	Prozessor:																			|
|	FuseBits: 																			|
|																						|
+---------------------------------------------------------------------------------------+
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "dmx.h"

volatile unsigned int dmx_adresse = 1;
volatile unsigned char dmx_toggle = DMX_LED_COUNT;
volatile unsigned char dmx_lost = 0;

//ACHTUNG: Diese Init Funktion ist fr den ATMega 8 geschrieben. Bei einem anderen Controller mssen die Register angepasst werden
void dmx_init(void)
{
	int baudrate;
	
	baudrate = UART_BAUD_SELECT(DMX_BAUD,F_CPU);
	
	//Baudrate in register schreiben
    UBRRH = (unsigned char)(baudrate>>8);
    UBRRL = (unsigned char) baudrate;

	UCSRB |= (1<<RXEN) | (1<<RXCIE);
	
	//8 Datenbit, 2 Stopbit, 
	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);

}

//############################################################################
//DMX Empfangsroutine von Ulrich Radig
ISR (USART_RECEIVE_VECTOR)
//############################################################################
{
	static unsigned int dmx_channel_rx_count = 0;
	static unsigned char dmx_valid = 0;
	static unsigned char dummy = 0;
	unsigned char tmp = 0;
	
	tmp =  UDR;
	
	if(UCSRA & (1<<FE))
	{
		if(dmx_channel_rx_count > 1)
		{
			//DMX lost
		}
		dmx_channel_rx_count = 0;	
		dummy = tmp;
		if(dummy == 0)
		{
			dmx_valid = 1;
			dmx_channel_rx_count++;
			
			//DMX Status LED
			dmx_toggle --;
			if((dmx_toggle == 0) && DMX_LED_ENABLE)
			{
				DMX_LED_PORT ^= (1<<DMX_LED_PIN);
				dmx_toggle = DMX_LED_COUNT;
			}
		}
		else
		{
			dmx_valid = 0;
		}
		return;
	}
	
	if(dmx_valid)
	{
		if((dmx_channel_rx_count >= dmx_adresse) &&  (dmx_channel_rx_count < (dmx_adresse + DMX_CHANNELS)))
		{
			dmx_buffer[dmx_channel_rx_count - dmx_adresse] = tmp;
		}

		//if(dmx_channel_rx_count == dmx_adresse) dmx_buffer[1] = tmp;
		//if(dmx_channel_rx_count == dmx_adresse+1) dmx_buffer[2] = tmp;
			
		if(dmx_channel_rx_count < 514)
		{
			dmx_channel_rx_count++;
		}
		
		return;
	}
}
