/*
+---------------------------------------------------------------------------------------+
|																						|
|	Author: Jonas Eberhard | www.joseng.de												|
|	Datei:	main.c																		|
|	Beschreibung: DMX-Audio Version 3 Firmware (Version mit TDA8425)					|
|																						|
|																						|
|	Prozessor:	ATmega 8																|
|	FuseBits: Low:0xFF  High: 0xD9														|
|																						|
+---------------------------------------------------------------------------------------+
*/

//Includes von externen Dateien
#include <avr/io.h>
#include <util/delay.h>  
#include <avr/interrupt.h>
//#include <avr/pgmspace.h>
//#include <stdlib.h>
#include <stdint.h>
//#include <string.h>
//#include <math.h>
//#include <stdio.h>
#include "dmx.h"
#include "twi.h"


//Quarz festlegen
#ifndef F_CPU
#warning "F_CPU war noch nicht definiert, wird nun mit 16MHz definiert"
#define F_CPU 16000000UL     /* Quarz mit 16 Mhz */
#endif

//Festgelegte Werte
#define TWI_ADR_TDA8425			130


//Funktionsprototypen
void init(void);


//Variablen deklarieren
uint8_t dmx_old_values[4] = {0,0,0,0};
uint8_t twi_start_send = 0;
uint8_t dummy = 0;


//Hauptprogramm
int main(void)
{
	init();
	while(1)
	{
		//Lautstrke links
		if(dmx_buffer[0] != dmx_old_values[0])
		{
			//I2C Wert fr die Lautstrke des Kanals berechnen
			dummy = 192 + 27 + (dmx_buffer[0] / 7);	//192 da die beiden MSB 1 sein mssen, 27 da der Offset zum ersten Schritt 27 ist
			
			//Neuer Wert ist alter Wert
			dmx_old_values[0] = dmx_buffer[0];
			
			if(twi_start(TWI_ADR_TDA8425))
			{
				twi_start_send = 1;
				//die Lautstrke links adressieren (geschieht mit erstem Byte das nach dem Start gesendet wird)
				twi_send_byte(0);
				//I2C Wert fr Lautstrke links senden
				twi_send_byte(dummy);
			}
		}
		
		//Lautstrke rechts
		if(dmx_buffer[1] != dmx_old_values[1])
		{
			//I2C Wert fr die Lautstrke des Kanals berechnen
			dummy = 192 + 27 + (dmx_buffer[1] / 7);	//192 da die beiden MSB 1 sein mssen, 27 da der Offset zum ersten Schritt 27 ist

			//Neuer Wert ist alter Wert
			dmx_old_values[1] = dmx_buffer[1];
			
			//Wenn noch kein Start gesendet wurde dann jetzt
			if(!twi_start_send)
			{
				//I2C Start senden
				if(twi_start(TWI_ADR_TDA8425))
				{
					twi_start_send = 1;
					//die Lautstrke rechts adressieren (geschieht mit erstem Byte das nach dem Start gesendet wird)
					twi_send_byte(1);
				}
			}
			//I2C Wert fr Lautstrke rechts senden
			twi_send_byte(dummy);
		}
		
		//Bass
		if(dmx_buffer[2] != dmx_old_values[2])
		{
			//I2C Wert fr Bassregelung errechnen
			dummy = 240 + 2 + (dmx_buffer[2] / 25);	//240 da die vier MSB 1 sein mssen, 2 ist Offset fr -12db, "/ 25" da der DMX Bereich in 10 Schritte unterteilt ist (255 / 10 = 25)
			
			//Neuer Wert ist alter Wert
			dmx_old_values[2] = dmx_buffer[2];
			
			//Wenn noch kein Start gesendet wurde dann jetzt
			if(!twi_start_send)
			{
				//I2C Start senden
				if(twi_start(TWI_ADR_TDA8425))
				{
					twi_start_send = 1;
					//die Bass adressieren (geschieht mit erstem Byte das nach dem Start gesendet wird)
					twi_send_byte(2);
				}
			}
			//I2C Wert fr Bass senden
			twi_send_byte(dummy);
		}
		
		//Hhen
		if(dmx_buffer[3] != dmx_old_values[3])
		{
			//I2C Wert fr Hhenregelung errechnen
			dummy = 240 + 2 + (dmx_buffer[3] / 28);	//240 da die vier MSB 1 sein mssen, 2 ist Offset fr -12db, "/ 28" da der DMX Bereich in 9 Schritte unterteilt ist (255 / 9 = 28)
			
			//Neuer Wert ist alter Wert
			dmx_old_values[3] = dmx_buffer[3];
			
			//Wenn noch kein Start gesendet wurde dann jetzt
			if(!twi_start_send)
			{
				//I2C Start senden
				if(twi_start(TWI_ADR_TDA8425))
				{
					twi_start_send = 1;
					twi_send_byte(3);
				}
			}
			//I2C Wert fr Hhen senden
			twi_send_byte(dummy);
		}
		
		//Wenn ein Start gesendet wurde muss auch ein Stop gesendet werden
		if(twi_start_send)
		{
			twi_start_send = 0;
			twi_stop();
		}
	}
}


//Init Routine
void init(void)
{
	//Hier hngt die Statusled
	DDRB = 0b00000001;
	PORTB = 0b00000000;

	//DMX Adresse festlegen
	dmx_adresse = 1;
	//DMX Uart Init
	dmx_init();

	//I2C Init mit 50kHz
	twi_init(50000);	
	
	//Interrupts freischalten
	sei();

	//TDA8425 Init, 
	_delay_ms(250);
	if(twi_start(TWI_ADR_TDA8425))
	{
		if(twi_send_byte(8))
		{
			twi_send_byte(0);		//Unteradresse fr Volume links ab da auto increment
			twi_send_byte(192);		//Volume links -80dB
			twi_send_byte(192);		//Volume rechts -80dB
			twi_send_byte(246);		//Bass 0dB
			twi_send_byte(246);		//Hhen 0dB
			twi_send_byte(206);		//hier wird der Source selector, Mute on Power und linear Stereo eingestellt
			twi_stop();
		}
	}
	
	
}