/*
+---------------------------------------------------------------------------------------+
|																						|
|	Author: Jonas Eberhard | www.joseng.de												|
|	Datei:	twi.h																		|
|	Beschreibung: I2C (TWI) Libary Header File											|
|																						|
|																						|
|	Prozessor:																			|
|	FuseBits: 																			|
|																						|
+---------------------------------------------------------------------------------------+
*/

//Maskierung des Statusregisters
#define TWI_STATUS (TWSR & 0b11111000)

//Master Start codes
#define TWI_START			0x08
#define TWI_REP_START		0x10


//Master TX Adresse/Daten Codes
#define TWI_MTX_ADR_ACK		0x18
#define TWI_MTX_ADR_NACK	0x20
#define TWI_MTX_DATA_ACK	0x28
#define TWI_MTX_DATA_NACK	0x30

//Master TX Kollision Code
#define TWI_MTX_ARB_LOST	0x38


//Master RX Adresse/Daten Codes
#define TWI_MRX_ADR_ACK		0x40
#define TWI_MRX_ADR_NACK	0x48
#define TWI_MRX_DATA_ACK	0x50
#define TWI_MRX_DATA_NACK	0x58

//Master RX Kollision Code
#define TWI_MRX_ARB_LOST	0x38


//Read Write defines fr Adressaufruf
#define TWI_WRITE	0
#define TWI_READ	1



//Funktionsprototypen
uint8_t twi_init(uint16_t);
uint8_t twi_start(uint8_t);
uint8_t twi_send_byte(uint8_t);
uint8_t twi_send_bytes(uint8_t, uint8_t *);
uint8_t twi_read_byte_ack(void);
uint8_t twi_read_byte_nack(void);
uint8_t twi_read_bytes(uint8_t, uint8_t *);
uint8_t twi_stop(void);
void blink(uint8_t);